# Copyright 2022 Soul Machines Ltd

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from typing import List

from smskillsdk.models.api import Memory
from smskillsdk.utils import memory


class TestMemoryUtils(unittest.TestCase):
    def test_serialize(self):
        memory_dict = {
            "key1": "value1",
            "key2": "value2",
            "key3": "value3",
        }

        memory_array = memory.serialize_memory(memory_dict)
        self.assertEqual(len(memory_array), 3)

        for i, memory_element in enumerate(memory_array):
            self.assertTrue(
                hasattr(memory_element, "name"),
                f"Memory element {i} is missing 'name' attribute",
            )
            self.assertTrue(
                hasattr(memory_element, "value"),
                f"Memory element {i} is missing 'value' attribute",
            )
            self.assertFalse(hasattr(memory_element, "session_id"))
            self.assertIn(memory_element.name, memory_dict)
            self.assertEqual(memory_element.value, memory_dict[memory_element.name])

    def test_serialize_with_session_id(self):
        memory_dict = {
            "key1": "value1",
            "key2": "value2",
            "key3": "value3",
        }
        session_id = "123"

        memory_array = memory.serialize_memory(memory_dict, session_id)
        self.assertEqual(len(memory_array), 3)

        for i, memory_element in enumerate(memory_array):
            self.assertTrue(
                hasattr(memory_element, "name"),
                f"Memory element {i} is missing 'name' attribute",
            )
            self.assertTrue(
                hasattr(memory_element, "value"),
                f"Memory element {i} is missing 'value' attribute",
            )
            self.assertTrue(hasattr(memory_element, "session_id"))
            self.assertIn(memory_element.name, memory_dict)
            self.assertEqual(memory_element.value, memory_dict[memory_element.name])
            self.assertEqual(memory_element.session_id, session_id)

    def test_deserialize(self):
        memory_array = [
            Memory(**{"name": "key1", "value": "value1"}),
            Memory(**{"name": "key2", "value": 1}),
            Memory(**{"name": "key3", "value": {"subkey2": "subvalue2"}}),
            Memory(**{"key": "key4", "value": "value"}),
        ]

        memory_dict = memory.deserialize_memory(memory_array)
        self.assertEqual(len(memory_dict), 3)

        for memory_element in memory_array:
            if hasattr(memory_element, "name") and hasattr(memory_element, "value"):
                self.assertIn(memory_element.name, memory_dict)
                self.assertEqual(memory_element.value, memory_dict[memory_element.name])

    def test_deserialize_with_session_id(self):
        memory_array = [
            Memory(**{"name": "key1", "value": "value1", "session_id": "123"}),
            Memory(**{"name": "key2", "value": 1}),
            Memory(
                **{
                    "name": "key3",
                    "value": {"subkey2": "subvalue2"},
                    "session_id": "123",
                }
            ),
        ]

        memory_dict = memory.deserialize_memory(memory_array, "123")
        self.assertEqual(set(memory_dict.keys()), {"key1", "key3"})

    def test_set_memory_value(self):
        memories: List[Memory] = list()

        memory.set_memory_value(memories, "key1", "value1")
        self.assertEqual(len(memories), 1)
        self.assertEqual(memories[0].name, "key1")
        self.assertEqual(memories[0].value, "value1")

        memory.set_memory_value(memories, "key1", "value1", "sessionid1")
        self.assertEqual(len(memories), 2)
        self.assertEqual(memories[-1].name, "key1")
        self.assertEqual(memories[-1].value, "value1")
        self.assertFalse(hasattr(memories[0], "session_id"))
        self.assertTrue(hasattr(memories[-1], "session_id"))

        memory.set_memory_value(memories, "key1", "value2")
        self.assertEqual(len(memories), 2)
        self.assertEqual(memories[0].name, "key1")
        self.assertEqual(memories[0].value, "value2")

        memory.set_memory_value(memories, "key1", "value3", "sessionid1")
        self.assertEqual(len(memories), 2)
        self.assertEqual(memories[0].name, "key1")
        self.assertEqual(memories[0].value, "value2")
        self.assertEqual(memories[-1].name, "key1")
        self.assertEqual(memories[-1].value, "value3")

        memory.set_memory_value(memories, "key2", "value2")
        self.assertEqual(len(memories), 3)
        self.assertEqual(memories[-1].name, "key2")
        self.assertEqual(memories[-1].value, "value2")

    def test_get_memory_value(self):
        memories: List[Memory] = list()

        self.assertEqual(memory.get_memory_value(memories, "key1")[0], False)

        memories.append(Memory(**{"name": "key1", "value": "value1"}))
        self.assertEqual(memory.get_memory_value(memories, "key1"), (True, "value1"))

        memories.append(
            Memory(**{"name": "key1", "value": "value2", "session_id": "sessionid1"})
        )
        self.assertEqual(memory.get_memory_value(memories, "key1"), (True, "value1"))
        self.assertEqual(
            memory.get_memory_value(memories, "key1", "sessionid1"), (True, "value2")
        )
        self.assertEqual(
            memory.get_memory_value(memories, "key1", "sessionid2")[0], False
        )


if __name__ == "__main__":
    unittest.main()



# Copyright 2022 Soul Machines Ltd

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# generated by datamodel-codegen:
#   filename:  skill-api.yml
#   timestamp: 2022-01-19T20:00:28+00:00

from __future__ import annotations

from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Extra, Field


class Intent(BaseModel):
    name: Optional[str] = Field(
        None,
        description="""Name of intent that handled the request or 'NO_MATCH' if a
            fallback is hit""",
    )
    confidence: Optional[float] = Field(
        None, description="Confidence of the intent matched in the range 0-1"
    )


class Variables(BaseModel):
    class Config:
        extra = Extra.allow

    public: Optional[Dict[str, Any]] = None


class Output(BaseModel):
    text: Optional[str] = None
    variables: Optional[Variables] = None


class Memory(BaseModel):
    class Config:
        extra = Extra.allow


class ExecuteRequest(BaseModel):
    projectId: str = Field(
        ..., description="(Future functionality) Salted hash of DDNA Studio Project ID"
    )
    sessionId: str = Field(
        ..., description="A unique identifier for this particular session"
    )
    intent: Optional[Intent] = None
    text: str = Field(..., description="User input")
    memory: List[Memory] = Field(
        ...,
        description="""Information stored from the most recent response received from
            the endpoint""",
    )
    context: Optional[Dict[str, Any]] = Field(
        None, description="Used to send DP specific data e.g. EQ data"
    )
    config: Optional[Dict[str, Any]] = Field(
        None,
        description="""Skill configuration data will be included in this object. This
            property will only be  included if the skill is stateless (i.e. no
            sessionEndpoint is provided).\n""",
    )


class ExecuteResponse(BaseModel):
    output: Optional[Output] = None
    intent: Optional[Intent] = None
    memory: Optional[List[Memory]] = None
    endConversation: bool = Field(
        ...,
        description="""Used to indicate whether the current interaction with the skill
            should be concluded.  Set to false if you want the skill to handle
            further conversation turns with the user.\n""",
    )


class SessionRequest(BaseModel):
    projectId: str = Field(
        ..., description="(Future functionality) Salted hash of DDNA Studio Project ID"
    )
    sessionId: str
    config: Optional[Dict[str, Any]] = None
    memory: Optional[List[Memory]] = None


class SessionResponse(BaseModel):
    memory: Optional[List[Memory]] = None



# Copyright 2022 Soul Machines Ltd

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utility functions for working with memory arrays"""
from typing import Any, Dict, List, Tuple, Union

from smskillsdk.models.api import Memory


def serialize_memory(data: dict, session_id: Union[str, None] = None) -> List[Memory]:
    """Convert a Python dict into a list of Memory objects

    Key-value pairs will be converted into objects with these properties

    {
        "name": <key: Any>,
        "value": <value: Any>
        "session_id": <session_id: str> (optional)
    }
    """
    memory = list()
    for key, value in data.items():
        if session_id is None:
            memory.append(Memory(**{"name": key, "value": value}))
        else:
            memory.append(
                Memory(**{"name": key, "value": value, "session_id": session_id})
            )

    return memory


def deserialize_memory(
    memories: List[Memory], session_id: Union[str, None] = None
) -> Dict[str, Any]:
    """Convert a list of memory objects to a dictionary

    It is assumed memory objects will have name/value attributes. If multiple memory
    objects share the same name values, the latest value in the list will be used. If
    a session_id is provided, this will be checked and used to filter out memories
    without a matching session_id.
    """
    result = dict()
    for memory in memories:
        if isinstance(memory, Memory):
            if session_id is not None and (
                not hasattr(memory, "session_id") or memory.session_id != session_id
            ):
                continue

            if hasattr(memory, "name") and hasattr(memory, "value"):
                result[memory.name] = memory.value

    return result


def get_memory_value(
    memories: List[Memory], key: str, session_id: Union[str, None] = None
) -> Tuple[bool, Any]:
    """Gets a value from the memory array corresponding to an optional session_id"""
    for memory in memories:
        if isinstance(memory, Memory):
            if session_id is not None and (
                not hasattr(memory, "session_id") or memory.session_id != session_id
            ):
                continue

            if (
                hasattr(memory, "name")
                and hasattr(memory, "value")
                and memory.name == key
            ):

                return True, memory.value

    return False, None


def set_memory_value(
    memories: List[Memory], key: str, value: Any, session_id: Union[str, None] = None
) -> None:
    """Sets a value in the memory array corresponding to an optional session_id"""
    found = False

    for memory in memories:
        if isinstance(memory, Memory):
            if session_id is not None and (
                not hasattr(memory, "session_id") or memory.session_id != session_id
            ):
                continue

            if (
                hasattr(memory, "name")
                and hasattr(memory, "value")
                and memory.name == key
            ):
                memory.value = value
                found = True

    if not found:
        if session_id is None:
            memories.append(Memory(**{"name": key, "value": value}))
        else:
            memories.append(
                Memory(**{"name": key, "value": value, "session_id": session_id})
            )
